/*
TCP Kodi

Change KODI_IP, SSID and PASSWORD.

Aktiver Player:
{"jsonrpc": "2.0", "method": "Player.GetActivePlayers", "id": 1}

Was wird in Player mit der id 1 gespielt (Video):
{"jsonrpc":"2.0","method":"Player.GetItem","params":{"playerid":1},"id":1}

Play/Pause Player mit id 1:
{"jsonrpc":"2.0","method":"Player.PlayPause","params":{"playerid":1},"id":1}

Nächstes Element im Player mit id 1:
{"jsonrpc": "2.0", "method": "Player.GoTo", "params": {"playerid":1, "to": "next"}, id":1}

Weitere Mögliche Befehle:,
http://kodi.wiki/view/JSON-RPC_API/v6
{"jsonrpc": "2.0", "method": "Player.GoTo", "params": {"playerid":0, "to": "previous"}, id":1}
{"jsonrpc": "2.0", "method": "System.Suspend", "id":1}
{"jsonrpc":"2.0","id":"1","method":"Player.Open","params":{"item":{"file":"special://profile/playlists/music/Morning.xsp"
*/

#define SSID "[Your SSID]"
#define PASSWORD "[Your Password]"
#define KODI_IP "192.168.179.22"

#define DEBUG true

#define LED_WLAN 13
#define BUTTON1 5
#define BUTTON2 10

#include <Arduino.h>
#include <SoftwareSerial.h>
#include <LiquidCrystal.h>

// RS, E, D4, D5, D6, D7
LiquidCrystal lcd(A0, A1, A2, A3, A4, A5);
SoftwareSerial esp8266(11, 12); // RX, TX

int id;
byte ticks;
String nowPlaying;

void setup()
{
  Serial.begin(19200);
  esp8266.begin(19200);

  lcd.begin(16, 2);
  lcd.print("KODI");

  if (!espConfig())
    serialDebug();
  else
    digitalWrite(LED_WLAN, HIGH);

  pinMode(BUTTON1, INPUT_PULLUP);
  pinMode(BUTTON2, INPUT_PULLUP);

  lcd.setCursor(0, 1);
  lcd.print("CONNECTED");
}

void loop()
{
  if (ticks == 0)
    updateNowPlaying();
  if ((ticks % 15) == 0)
    scrollLCD();
  if ((ticks % 10) == 0)
    checkButtons();
  delay(10);
  ticks++;
}

//-------------------
void updateNowPlaying()
{
  nowPlaying = getPlaying();
  lcd.clear();
  lcd.print("Playing:");
  lcd.setCursor(0, 1);
  lcd.print(nowPlaying);
}

void scrollLCD()
{
  int positionCounter;
  if (nowPlaying.length() > 16)
    lcd.scrollDisplayLeft();
}

void checkButtons()
{
  if (!digitalRead(BUTTON1))
  {
    playPause();
  }
  if (!digitalRead(BUTTON2))
  {
    playNext();
  }
}

// ---------------------------
bool sendKodiJsonPost(String method, String parameter)
{
  return sendTcpPost(KODI_IP, "8080", "/jsonrpc", "Content-Type: application/json", "{\"jsonrpc\": \"2.0\", \"method\": \"" + method + "\",\"params\":" + parameter + ", \"id\": 1}");
}

bool sendKodiJsonPost(String method)
{
  return sendTcpPost(KODI_IP, "8080", "/jsonrpc", "Content-Type: application/json", "{\"jsonrpc\": \"2.0\", \"method\": \"" + method + "\", \"id\": 1}");
}

bool sendTcpPost(String host, String port, String subpage, String parameter, String data)
{
  String request = "POST " + subpage + " HTTP/1.1\r\nHost:" + host + "\r\nContent-Length: " + String(data.length()) + "\r\n" + parameter + "\r\n\r\n" + data;

  if (sendCom("AT+CIPSTART=\"TCP\",\"" + host + "\",8080", "OK"))
  {
    if (sendCom("AT+CIPSEND=" + String(request.length() + 2), ">"))
    {
      esp8266.println(request);
      return true;
    }
  }
  return false;
}

// -------------- KODI Control Functions ------------------------
int getPlayerID()
{
  int id = -1;
  if (sendKodiJsonPost("Player.GetActivePlayers"))
  {
    if (esp8266.find("result"))
    {
      if (esp8266.findUntil("playerid\":", "]"))
      {
        id = esp8266.parseInt();
      }
    }
  }
  debug(String(id));
  cleanClose();
  return id;
}

String getPlaying()
{
  String label = " - ";
  id = getPlayerID();
  if (id != -1)
  {
    if (sendKodiJsonPost("Player.GetItem", "{\"playerid\":" + String(id) + "}"))
    {
      if (esp8266.find("label\":\""))
      {
        label = esp8266.readStringUntil('\"');
        debug(label);
        if (label.length() > 32)
          label = label.substring(0, 32);
      }
    }
  }
  cleanClose();
  return label;
}

void playPause()
{
  debug("Play/Pause");

  if (id != -1)
  {
    lcd.clear();
    lcd.print("  Play/Pause");
    lcd.setCursor(0, 1);

    sendKodiJsonPost("Player.PlayPause", "{\"playerid\":" + String(id) + "}");
    cleanClose();
  }
  while (!digitalRead(BUTTON1));
}

void playNext()
{
  debug("Play Next");
  if (id != -1)
  {
    lcd.clear();
    lcd.print("  Next");
    lcd.setCursor(0, 1);

    sendKodiJsonPost("Player.GoTo", "{\"playerid\":" + String(id) + ",\"to\": \"next\"}");
    cleanClose();
  }
  while (!digitalRead(BUTTON2));
  delay(2000);
}

//-----------------------------------------Config ESP8266------------------------------------

boolean espConfig()
{
  boolean success = true;
  esp8266.setTimeout(5000);
  success &= sendCom("AT+RST", "ready");
  esp8266.setTimeout(1000);
  if (configStation(SSID, PASSWORD))
  {
    success &= true;
    debug("WLAN Connected");
    debug("My IP is:");
    debug(sendCom("AT+CIFSR"));
  }
  else
  {
    success &= false;
  }
  //shorter Timeout for faster wrong UPD-Comands handling
  success &= sendCom("AT+CIPMODE=0", "OK"); //So rum scheit wichtig!
  success &= sendCom("AT+CIPMUX=0", "OK");

  return success;
}

boolean configStation(String vSSID, String vPASSWORT)
{
  boolean success = true;
  success &= (sendCom("AT+CWMODE=1", "OK"));
  esp8266.setTimeout(20000);
  success &= (sendCom("AT+CWJAP=\"" + String(vSSID) + "\",\"" + String(vPASSWORT) + "\"", "OK"));
  esp8266.setTimeout(1000);
  return success;
}

//-----------------------------------------------Controll ESP-----------------------------------------------------

void cleanClose()
{
  while (esp8266.read() >= 0); //empty read buffer
  sendCom("AT+CIPCLOSE");
}

boolean sendCom(String command, char respond[])
{
  esp8266.println(command);
  if (esp8266.findUntil(respond, "ERROR"))
  {
    return true;
  }
  else
  {
    debug("ESP SEND ERROR: " + command);
    return false;
  }
}

String sendCom(String command)
{
  esp8266.println(command);
  return esp8266.readString();
}

//-------------------------------------------------Debug Functions------------------------------------------------------
void serialDebug()
{
  while (true)
  {
    if (esp8266.available())
      Serial.write(esp8266.read());
    if (Serial.available())
      esp8266.write(Serial.read());
  }
}

void debug(String Msg)
{
  if (DEBUG)
  {
    Serial.println(Msg);
  }
}
